input[type=text],
input[type=file],
input[type=password],
input[type=number],
input[type=url],
input[type=email],
textarea {
	background-color: $bg-primary;
	display: block;
	width: 100%;

	td & {
		display: inline-block;
		width: auto;
	}
}

input[type=text],
input[type=file],
input[type=password],
input[type=number],
input[type=url],
input[type=email] {
	line-height: 38px;
	text-indent: 10px;
	margin-top: 10px;
}

input[type=file] {
	padding: 12px;
	line-height: 1;
	text-indent: 0;
	outline: none;
}

input[type=radio] {
	display: none;

	& + label {
		position: relative;
		padding-left: 30px;
		color: #191919;
		font-weight: 200;
		font-size: 16px;
		margin-top: 3px;
	}

	& + label:before {
		position: absolute;
		top: 0;
		left: 0;
		content: '';
		border-radius: 50%;
		border: 2px solid #191919;
		width: 14px;
		height: 14px;
		margin-right: 30px;
	}

	&:checked + label:after {
		position: absolute;
		top: 0;
		left: 0;
		content: '';
		width: 18px;
		height: 18px;
		background-color: #0084ff;
		border-radius: 50%;
		margin-right: 30px;
	}
}

input[type=checkbox] {
  position: relative;

  &:after{
    content: '';
    position: absolute;
    top: 50%; left: 50%;
    width: 70px; height: 35px;
    background: rgba($color: #ff00ff, $alpha: 0);
    transform: translate(-50%, -50%);
  }
}

legend {
	@include heading-large;
}

label {
	color: $gray-dark;
	font-size: 18px;
	font-weight: normal;
	display: block;
	margin-bottom: 15px;

	&.required {
		position: relative;
		padding-left: 11px;

		&:before {
			content: '*';
			position: absolute;
			left: 0;
			top: 0;
			font-weight: bold;
		}
	}
}

fieldset {
	&:not(:last-of-type),
	& > div:not(:last-of-type) {
		margin-bottom: 40px;
	}
}

.field-row {
	margin-bottom: 25px;

	em {
		margin-bottom: 10px;
		display: inline-block;
	  	width: 100%;
	}
}

.object-name {
	font-weight: bold;
}

// cannot find markup to update classname
.errorlist {
	margin: -15px 0 25px;
	padding: 0;

	li {
		color: $red;
	}
}

.errornote{
  color: $red;
}

.messagelist {
	@include heading-medium;
	color: #0084ff;
	margin-bottom: 20px;
}
