# Generated by Django 2.2.11 on 2020-03-30 22:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0002_auto_20150507_1728'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cmslog',
            name='action',
            field=models.PositiveIntegerField(choices=[(0, 'Save'), (2, 'Delete'), (3, 'Published'), (4, 'Unpublished'), (5, 'Scheduled')]),
        ),
    ]
