.actions {

  &__action {
    display: inline-block;
    width: 12px;
    height: 12px;
    margin-right: 20px;

    .fa {
      line-height: 0;
      color: $black;
    }

    &:last-of-type {
      margin-right: 0;
    }

    &:hover {

      .fa {
        color: $red;
      }
    }
  }

  &__help {
    font-size: 12px;
    margin-top: 8px;
  }
}
