{% extends "cms/toolbar.html" %}

{% block annotation_command %}
  <a class="disabled" data-wysihtml-command="insertAnnotation">annotate</a>
{% endblock %}

{% block annotation_modal %}
  <div class="editor__modal" data-wysihtml-dialog="insertAnnotation" style="display: none;">
    <label>Enter annotation</label>
    <input type="hidden" data-wysihtml-dialog-field="href">
    <input type="hidden" data-wysihtml-dialog-field="class" value="text--annotated">
    <textarea class="text"></textarea>
    <div class="button-group button-group--submit">
      <a data-wysihtml-dialog-action="cancel" class="button button--secondary">Cancel</a>
      <a data-wysihtml-dialog-action="save" class="button button--primary">OK</a>
    </div>
  </div>
{% endblock %}
